package javax.naming.spi;

import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.NamingEnumeration;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.Attributes;
import javax.naming.Context;
import javax.naming.CompositeName;
import javax.naming.NamingException;
import javax.naming.Name;
import java.util.Hashtable;
import javax.naming.CannotProceedException;
import javax.naming.directory.DirContext;

class ContinuationDirContext extends ContinuationContext implements DirContext
{
    ContinuationDirContext(CannotProceedException ex, Hashtable<?, ?> hashtable) {
        super(ex, hashtable);
    }
    
    protected DirContextNamePair getTargetContext(Name name) throws NamingException {
        if (this.cpe.getResolvedObj() == null) {
            throw (NamingException)this.cpe.fillInStackTrace();
        }
        Context context = NamingManager.getContext(this.cpe.getResolvedObj(), this.cpe.getAltName(), this.cpe.getAltNameCtx(), this.env);
        if (context == null) {
            throw (NamingException)this.cpe.fillInStackTrace();
        }
        if ((context instanceof DirContext)) {
            return new DirContextNamePair((DirContext)context, name);
        }
        if ((context instanceof Resolver)) {
            ResolveResult resolveToClass = ((Resolver)context).resolveToClass(name, DirContext.class);
            return new DirContextNamePair((DirContext)resolveToClass.getResolvedObj(), resolveToClass.getRemainingName());
        }
        Object lookup = context.lookup(name);
        if ((lookup instanceof DirContext)) {
            return new DirContextNamePair((DirContext)lookup, new CompositeName());
        }
        throw (NamingException)this.cpe.fillInStackTrace();
    }
    
    protected DirContextStringPair getTargetContext(String s) throws NamingException {
        if (this.cpe.getResolvedObj() == null) {
            throw (NamingException)this.cpe.fillInStackTrace();
        }
        Context context = NamingManager.getContext(this.cpe.getResolvedObj(), this.cpe.getAltName(), this.cpe.getAltNameCtx(), this.env);
        if ((context instanceof DirContext)) {
            return new DirContextStringPair((DirContext)context, s);
        }
        if ((context instanceof Resolver)) {
            ResolveResult resolveToClass = ((Resolver)context).resolveToClass(s, DirContext.class);
            DirContext dirContext = (DirContext)resolveToClass.getResolvedObj();
            Name remainingName = resolveToClass.getRemainingName();
            return new DirContextStringPair(dirContext, (remainingName != null) ? remainingName.toString() : "");
        }
        Object lookup = context.lookup(s);
        if ((lookup instanceof DirContext)) {
            return new DirContextStringPair((DirContext)lookup, "");
        }
        throw (NamingException)this.cpe.fillInStackTrace();
    }
    
    @Override
    public Attributes getAttributes(String s) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().getAttributes(targetContext.getString());
    }
    
    @Override
    public Attributes getAttributes(String s, String[] array) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().getAttributes(targetContext.getString(), array);
    }
    
    @Override
    public Attributes getAttributes(Name name) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().getAttributes(targetContext.getName());
    }
    
    @Override
    public Attributes getAttributes(Name name, String[] array) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().getAttributes(targetContext.getName(), array);
    }
    
    @Override
    public void modifyAttributes(Name name, int n, Attributes attributes) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        targetContext.getDirContext().modifyAttributes(targetContext.getName(), n, attributes);
    }
    
    @Override
    public void modifyAttributes(String s, int n, Attributes attributes) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        targetContext.getDirContext().modifyAttributes(targetContext.getString(), n, attributes);
    }
    
    @Override
    public void modifyAttributes(Name name, ModificationItem[] array) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        targetContext.getDirContext().modifyAttributes(targetContext.getName(), array);
    }
    
    @Override
    public void modifyAttributes(String s, ModificationItem[] array) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        targetContext.getDirContext().modifyAttributes(targetContext.getString(), array);
    }
    
    @Override
    public void bind(Name name, Object o, Attributes attributes) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        targetContext.getDirContext().bind(targetContext.getName(), o, attributes);
    }
    
    @Override
    public void bind(String s, Object o, Attributes attributes) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        targetContext.getDirContext().bind(targetContext.getString(), o, attributes);
    }
    
    @Override
    public void rebind(Name name, Object o, Attributes attributes) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        targetContext.getDirContext().rebind(targetContext.getName(), o, attributes);
    }
    
    @Override
    public void rebind(String s, Object o, Attributes attributes) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        targetContext.getDirContext().rebind(targetContext.getString(), o, attributes);
    }
    
    @Override
    public DirContext createSubcontext(Name name, Attributes attributes) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().createSubcontext(targetContext.getName(), attributes);
    }
    
    @Override
    public DirContext createSubcontext(String s, Attributes attributes) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().createSubcontext(targetContext.getString(), attributes);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes attributes, String[] array) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().search(targetContext.getName(), attributes, array);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, Attributes attributes, String[] array) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().search(targetContext.getString(), attributes, array);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes attributes) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().search(targetContext.getName(), attributes);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, Attributes attributes) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().search(targetContext.getString(), attributes);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String s, SearchControls searchControls) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().search(targetContext.getName(), s, searchControls);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, String s2, SearchControls searchControls) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().search(targetContext.getString(), s2, searchControls);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String s, Object[] array, SearchControls searchControls) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().search(targetContext.getName(), s, array, searchControls);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, String s2, Object[] array, SearchControls searchControls) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().search(targetContext.getString(), s2, array, searchControls);
    }
    
    @Override
    public DirContext getSchema(String s) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().getSchema(targetContext.getString());
    }
    
    @Override
    public DirContext getSchema(Name name) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().getSchema(targetContext.getName());
    }
    
    @Override
    public DirContext getSchemaClassDefinition(String s) throws NamingException {
        DirContextStringPair targetContext = this.getTargetContext(s);
        return targetContext.getDirContext().getSchemaClassDefinition(targetContext.getString());
    }
    
    @Override
    public DirContext getSchemaClassDefinition(Name name) throws NamingException {
        DirContextNamePair targetContext = this.getTargetContext(name);
        return targetContext.getDirContext().getSchemaClassDefinition(targetContext.getName());
    }
}
